from ctypes import *
from dwfconstants import *
import sys
import time
import math
import numpy as np

"""
トラ技2023年3月号ダウンロードサービス
「ラズパイを使ったポータブル万能計測ステーション」 
"""

if sys.platform.startswith("win"):
    dwf = cdll.LoadLibrary("dwf.dll")
elif sys.platform.startswith("darwin"):
    dwf = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/dwf")
else:
    dwf = cdll.LoadLibrary("libdwf.so")
hdwf = c_int()
# デバイスオープン FDwfDeviceOpen(int idxDevice, HDWF *phdwf)
# hdwf はハンドルID で今後第一引数で使われてゆく
dwf.FDwfDeviceOpen(c_int(-1), byref(hdwf))
if hdwf.value == 0:
    print("Failed to open device")
    sys.exit()

# +5Vユーザー電源ON (V+)
dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(0), c_int(0), c_double(1))
dwf.FDwfAnalogIOEnableSet(hdwf, c_bool(True))

# ↓ 方形波 Duty比 v.s. 出力電圧特性 -------------------------------
# 設定パラメーター変更しても、直ぐに反映しない設定。
# xxxConfigure( ) 系関数により、設定反映させる。
dwf.FDwfDeviceAutoConfigureSet(hdwf, c_int(0))

# 波形発生器 CN1 設定 (W1)
w1_channel = c_int(0)
w2_channel = c_int(1)
# キャリアON, キャリア方形波
dwf.FDwfAnalogOutNodeEnableSet(hdwf, w1_channel, AnalogOutNodeCarrier, c_bool(True))
dwf.FDwfAnalogOutNodeFunctionSet(hdwf, w1_channel, AnalogOutNodeCarrier, funcSquare)
# 周波数 500Hz, 振幅 1.65V, オフセット 1.65V (= 0/3.3V 方形波), リピート 無限
dwf.FDwfAnalogOutNodeFrequencySet(hdwf, w1_channel, AnalogOutNodeCarrier, c_double(500.0))
dwf.FDwfAnalogOutNodeAmplitudeSet(hdwf, w1_channel, AnalogOutNodeCarrier, c_double(1.65))
dwf.FDwfAnalogOutNodeOffsetSet(hdwf, w1_channel, AnalogOutNodeCarrier, c_double(1.65))
dwf.FDwfAnalogOutRepeatSet(hdwf, w1_channel, c_int(0))

# オシロスコープ CH2 設定 (2+/2-)
sc1_channel = c_int(0)
sc2_channel = c_int(1)
vout2 = c_double()
# Scope2 有効, 電圧レンジ -0.5 to 4.5 [V] (= +-5/2 + 2)
dwf.FDwfAnalogInChannelEnableSet(hdwf, sc2_channel, c_bool(True))
dwf.FDwfAnalogInChannelRangeSet(hdwf, sc2_channel, c_double(5))
dwf.FDwfAnalogInChannelOffsetSet(hdwf, sc2_channel, c_double(2))
dwf.FDwfAnalogInAcquisitionModeSet(hdwf, acqmodeSingle)
time.sleep(2)
dwf.FDwfAnalogInConfigure(hdwf, c_bool(False), c_bool(False))  # 設定値反映
#  Duty 0 - 100 まで 10 [%] 刻みで掃引
print('Duty[%], Voltage[V]')
for i in range(11):
    dwf.FDwfAnalogOutNodeSymmetrySet(hdwf, w1_channel, AnalogOutNodeCarrier, c_double(i*10))
    dwf.FDwfAnalogOutConfigure(hdwf, w1_channel, c_bool(True))  # 設定値反映
    time.sleep(1)
    dwf.FDwfAnalogInStatus(hdwf, c_bool(True), None)  # データ取得
    dwf.FDwfAnalogInStatusSample(hdwf, sc2_channel, byref(vout2))  # 取得されたデータ取込み
    print('{}, {}'.format(i*10, vout2.value))
# ↑ 方形波 Duty比 v.s. 出力電圧特性 -------------------------------

# ↓ FFT Duty 50% ------------------------------------------------
sample_freq = 20e3
n_samples = 8192
record = (c_double*n_samples)()
status = c_ubyte()

# 50% Duty 時の出力電圧測定
dwf.FDwfAnalogOutNodeSymmetrySet(hdwf, w1_channel, AnalogOutNodeCarrier, c_double(50))
dwf.FDwfAnalogOutConfigure(hdwf, w1_channel, c_bool(True))
time.sleep(1)
dwf.FDwfAnalogInStatus(hdwf, c_bool(True), None)
dwf.FDwfAnalogInStatusSample(hdwf, sc2_channel, byref(vout2))
# Scope2 レンジ設定: 先の出力電圧オフセットの、20mV レンジ
dwf.FDwfAnalogInChannelRangeSet(hdwf, sc2_channel, c_double(20e-3))
dwf.FDwfAnalogInChannelOffsetSet(hdwf, sc2_channel, vout2)
time.sleep(2)
dwf.FDwfAnalogInConfigure(hdwf, c_bool(False), c_bool(False))  # 設定値反映
# サンプリング周波数 20kHz, サンプル点数 8192 設定
dwf.FDwfAnalogInFrequencySet(hdwf, c_double(sample_freq))
dwf.FDwfAnalogInBufferSizeSet(hdwf, c_int(n_samples))
time.sleep(2)
dwf.FDwfAnalogInConfigure(hdwf, c_bool(False), c_bool(True))  # 設定値反映 & サンプリング開始
while True:
    dwf.FDwfAnalogInStatus(hdwf, c_bool(True), byref(status))  # サンプリング中か? 確認
    if status.value == DwfStateDone.value:  # サンプリング完了ならループ抜ける
        break
    time.sleep(0.1)
# サンプリング終了したので、データ取込み
dwf.FDwfAnalogInStatusData(hdwf, sc2_channel, record, c_int(n_samples))
# list(record) -> 20ksps sampling, 8192 point
osc_data = np.array(list(record))
fft_data = np.fft.fft(osc_data)
fft_freq = np.fft.fftfreq(n_samples, d=1/sample_freq)
amplog_data = 20*np.log10(np.abs(fft_data/(n_samples/2)))
np.save('./fft', amplog_data[:n_samples//2-1])
# ↑ FFT Duty 50% ------------------------------------------------

# ↓ 周波数 v.s. 振幅特性 -----------------------------------------
amp1 = c_double()
amp2 = c_double()
phase1 = c_double()
phase2 = c_double()
start_freq = 1
list_freq = [1, 2, 3, 5, 8, 10, 20, 30, 50, 80,
             100, 200, 300, 500, 800,
             1000, 2000]
# 設定パラメーター変更したら、すぐ反映
dwf.FDwfDeviceAutoConfigureSet(hdwf, c_int(3))
dwf.FDwfAnalogImpedanceReset(hdwf)
# W1, OSC1(1+/1-) を入力端に、OSC2(2+/2-) を出力端に接続するモード
dwf.FDwfAnalogImpedanceModeSet(hdwf, c_int(0))
dwf.FDwfAnalogImpedanceReferenceSet(hdwf, c_double(1e6))  # dummy
dwf.FDwfAnalogImpedanceFrequencySet(hdwf, c_double(list_freq[0]))
# Input: 下限0.6, 上限2.6 [V] の正弦波
dwf.FDwfAnalogImpedanceAmplitudeSet(hdwf, c_double(1))
dwf.FDwfAnalogImpedanceOffsetSet(hdwf, c_double(0.6))
# スタート
dwf.FDwfAnalogImpedanceConfigure(hdwf, c_int(1))
time.sleep(2)
print('Freq[Hz], Gain[dB], Phase[degree]')
for freq in list_freq:
    dwf.FDwfAnalogImpedanceFrequencySet(hdwf, c_double(freq))
    time.sleep(0.01)
    # 周波数変更したので、前のサンプリングデータを無視して、またサンプリング始める
    dwf.FDwfAnalogImpedanceStatus(hdwf, None)
    while True:
        dwf.FDwfAnalogImpedanceStatus(hdwf, byref(status))   # サンプリング中か? 確認
        if status.value == DwfStateDone.value:  # サンプリング完了ならループ抜ける
            break
        time.sleep(0.1)
    # dwf.FDwfAnalogImpedanceStatusInputは、w1 の波形を基準とした、利得・位相が得られる。
    # よって以下の amp2 = w1/w2 (= 1/Gain), phase2 = w1phase - w2phase (= -Phase) になる。
    # 利得は逆数、位相は正負反転になる点に注意
    dwf.FDwfAnalogImpedanceStatusInput(hdwf, w2_channel, byref(amp2), byref(phase2))
    gain = -20 * math.log10(amp2.value)  # amp2は利得逆数なので、マイナスを付けて利得に戻す
    phase = -180/math.pi*phase2.value    # phase2は正負反転し、ラジアンから度に単位変換
    phase = phase if phase < 0 else phase-360  # 位相を0 から -360度表現に変換
    print('{}, {}, {}'.format(freq, gain, phase))
# ↑ 周波数 v.s. 振幅特性 -----------------------------------------

dwf.FDwfDeviceClose(hdwf)  # Close Device
